@@ -1,6 +1,13 @@
+"""This module takes care of patching a few methods from various
+IDLE classes at import time."""
+
 from friendly_idle.patching_hook import add_patch
 
 
+# In normal usage, idlelib.run is imported multiple times
+# during a normal session with IDLE.
+
+
 def add_recreate_syntax_error(module):
     """Adds a method to idlelib.run.Executive to recreate
     a SyntaxError or other errors occurring at compile time
@@ -25,6 +32,7 @@ def add_recreate_syntax_error(module):
 
 
 add_patch("idlelib.run", add_recreate_syntax_error)
+# ========================================================================
 
 
 def replace_transfer_path(module):
@@ -41,6 +49,7 @@ def replace_transfer_path(module):
         else:
             path = sys.path
 
+        # Add required info for friendly
         self.runcommand(
             """if 1:
         import sys as _sys
@@ -52,33 +61,48 @@ def replace_transfer_path(module):
         \n"""
             % (path, __file__)
         )
+        # Also, introduce a variable to fix the issue noted by Raymond Hettinger
+        # in https://twitter.com/raymondh/status/1501700903870468099
+        self.error_at_end_of_line = False
 
     module.ModifiedInterpreter.transfer_path = transfer_path
     return module
 
 
 add_patch("idlelib.pyshell", replace_transfer_path)
+# ========================================================================
 
 
 def replace_idle_title(module):
     """Replaces IDLE's title, prepending it by Friendly"""
     from platform import python_version
     from friendly_idle import __version__
-    module.PyShell.shell_title = f"Friendly IDLE (version {__version__}; Python version {python_version()})"
+
+    module.PyShell.shell_title = (
+        f"Friendly IDLE (version {__version__}; Python version {python_version()})"
+    )
     return module
 
 
 add_patch("idlelib.pyshell", replace_idle_title)
+# ========================================================================
 
 
 def replace_runsource(module):
-    """Replaces idlelib.pyshell.
+    """Replaces idlelib.pyshell.runsource so that we save the source code
+    entered in the shell, together with the 'filename' which would be of
+    the form '<pyshell#...>'.
     """
 
     def runsource(self, source):
         "Extend base class method: Stuff the source in the line cache first"
         from code import InteractiveInterpreter
 
+        if self.error_at_end_of_line:
+            # For problem reported by R. Hettinger on Twitter.
+            self.error_at_end_of_line = False
+            self.tkconsole.text.tag_remove("ERROR", "1.0", "end")
+
         filename = self.stuffsource(source)
 
         self._source = source
@@ -91,12 +115,13 @@ def replace_runsource(module):
 
 
 add_patch("idlelib.pyshell", replace_runsource)
+# ========================================================================
 
 
 def replace_showsyntaxerror(module):
     """Replaces idlelib.pyshell.ModifiedInterpreter.showsyntaxerror
-    adding a call to a function used to recreate the SyntaxError
-    and process it with friendly.
+    adding a call to a function added to idlelib.run.Executive and
+    used to recreate the SyntaxError and process it with friendly-traceback.
     """
 
     def showsyntaxerror(self, filename=None):
@@ -120,6 +145,9 @@ def replace_showsyntaxerror(module):
             pos = "iomark + %d chars" % (offset - 1)
         else:
             pos = "iomark linestart + %d lines + %d chars" % (lineno - 1, offset - 1)
+
+        # For problem reported by R. Hettinger on Twitter.
+        self.error_at_end_of_line = text.get(pos) == "\n"
         tkconsole.colorize_syntax_error(text, pos)
         tkconsole.resetoutput()
 
@@ -134,7 +162,8 @@ def replace_showsyntaxerror(module):
         # highlighted in red, and a second time (in the friendly-traceback) with
         # carets (^) indicating the location of the error.
         # The approach we use is to *compile* the code with the repeated filename
-        # and explicitly catch the exception with a customized display.
+        # and explicitly catch the exception in the part of the code
+        # where it will be executed..
 
         if self.rpcclt:
             self.rpcclt.remotequeue(
@@ -149,6 +178,7 @@ def replace_showsyntaxerror(module):
 
 
 add_patch("idlelib.pyshell", replace_showsyntaxerror)
+# ========================================================================
 
 
 def replace_build_subprocess_arglist(module):
@@ -180,11 +210,14 @@ def replace_build_subprocess_arglist(module):
 
 
 add_patch("idlelib.pyshell", replace_build_subprocess_arglist)
+# ========================================================================
 
 
 def replace_checksyntax(module):
     """Replaces idlelib.runscript.ScriptBinding.checksyntax so that
     the information from friendly-traceback can be shown in the popup window.
+    This method is called either when a menu item is used to check the syntax
+    or to actually run a module.
     """
 
     def checksyntax(self, filename):
@@ -192,6 +225,7 @@ def replace_checksyntax(module):
         from friendly import idle_writer
         from functools import partial
 
+        # Most of the code below is recopied as is from IDLE.
         self.shell = shell = self.flist.open_shell()
         saved_stream = shell.get_warning_stream()
         shell.set_warning_stream(shell.stderr)
@@ -216,6 +250,7 @@ def replace_checksyntax(module):
                 lineno += 1  # mark end of offending line
             pos = "0.0 + %d lines + %d chars" % (lineno - 1, offset - 1)
             editwin.colorize_syntax_error(text, pos)
+            # New code, to prepare the new try block.
             _writer = partial(idle_writer.writer, stream=shell)
             _formatter = idle_writer.formatter
             try:
@@ -226,6 +261,7 @@ def replace_checksyntax(module):
             except Exception as exc:
                 print("Attempting to process SyntaxError with friendly failed.")
                 print(exc)
+                # Normally, this would be the only line executed by IDLE.
                 self.errorbox("SyntaxError", "%-20s" % msg)
             return False
         finally:
